#include <QQmlApplicationEngine>
#include <QQmlContext>

#include "dynamicclient.hpp"

// c'tor
DynamicClient::DynamicClient(QRemoteObjectNode& repNode, QQmlApplicationEngine& engine) :
    QObject(nullptr), m_engine (engine)
{
//    _ptrTestModel.reset(repNode.acquireModel(QStringLiteral("TestModel"), QtRemoteObjects::PrefetchData));
    m_ptrTestModel.reset(repNode.acquireModel(QStringLiteral("TestModel"), QtRemoteObjects::PrefetchData));
    m_ptrTestControler.reset(repNode.acquireDynamic(QStringLiteral("TestControler")));

    m_pTestHorzHeader = new AQmlRemoteObjectHelperHeaderModel(this, m_ptrTestModel, Qt::Horizontal);

    //connect signal for replica valid changed with signal slot initialization
    QObject::connect(m_ptrTestModel.data(), SIGNAL(initialized()), this, SLOT(init_slot()));
    QObject::connect(m_ptrTestControler.data(), SIGNAL(initialized()), this, SLOT(init_slot()));
}

//d'tor
DynamicClient::~DynamicClient()
{
}

void DynamicClient::init_slot()
{
    static bool testModelInitialized{false};
    static bool testControlerInitialized{false};

    if( sender() )
    {
        if( sender() == m_ptrTestModel.data() )
        {
            testModelInitialized = true;
            qDebug() << QString("TestModelInitialized");
            m_engine.rootContext()->setContextProperty("testmodel", m_ptrTestModel.data());
            m_engine.rootContext()->setContextProperty("testmodelHorzHeader", m_pTestHorzHeader);
        }
        else if(sender() == m_ptrTestControler.data())
        {
            testControlerInitialized = true;
            qDebug() << QString("TestControlerInitialized");
            m_engine.rootContext()->setContextProperty("testcontroler", m_ptrTestControler.data());
        }
    }

    if( testModelInitialized && testControlerInitialized )
    {
        m_engine.load(QUrl(QStringLiteral("qrc:/main.qml")));
        qDebug() << QString("QML Engine loaded ");
    }
}




AQmlRemoteObjectHelperHeaderModel::AQmlRemoteObjectHelperHeaderModel(QObject* pParent, QSharedPointer<QAbstractItemModelReplica> ptr, Qt::Orientation ori)
      : QAbstractListModel(pParent), m_parentModel(ptr), m_orientation(ori)
{
    QObject::connect(m_parentModel.data(), SIGNAL(headerDataChanged(Qt::Orientation, int, int)), this, SLOT(handleHeaderDataChanged_slot(Qt::Orientation, int, int)));
//    QObject::connect(_pParentModel.data(), SIGNAL(modelReset()), this, SIGNAL(modelReset()));
    QObject::connect(m_parentModel.data(), &AQmlRemoteObjectHelperHeaderModel::modelAboutToBeReset, this, &AQmlRemoteObjectHelperHeaderModel::handleBeginReset_slot);
    QObject::connect(m_parentModel.data(), &AQmlRemoteObjectHelperHeaderModel::modelReset, this, &AQmlRemoteObjectHelperHeaderModel::handleEndReset_slot);
    if( Qt::Vertical == m_orientation)
    {
        QObject::connect(m_parentModel.data(), &AQmlRemoteObjectHelperHeaderModel::rowsAboutToBeInserted, this, [this](){this->beginResetModel();});
        QObject::connect(m_parentModel.data(), &AQmlRemoteObjectHelperHeaderModel::rowsInserted, this, [this](){this->endResetModel();});
        QObject::connect(m_parentModel.data(), &AQmlRemoteObjectHelperHeaderModel::rowsAboutToBeRemoved, this, [this](){this->beginResetModel();});
        QObject::connect(m_parentModel.data(), &AQmlRemoteObjectHelperHeaderModel::rowsRemoved, this, [this](){this->endResetModel();});
        QObject::connect(m_parentModel.data(), &AQmlRemoteObjectHelperHeaderModel::rowsAboutToBeMoved, this, [this](){this->beginResetModel();});
        QObject::connect(m_parentModel.data(), &AQmlRemoteObjectHelperHeaderModel::rowsMoved, this, [this](){this->endResetModel();});
//        QObject::connect(_pParentModel.data(), &AQmlRemoteObjectHelperHeaderModel::modelAboutToBeReset, this, [this](){beginResetModel();});
//        QObject::connect(_pParentModel.data(), &AQmlRemoteObjectHelperHeaderModel::modelReset, this, [this](){endResetModel();});
//        QObject::connect(_pParentModel.data(), SIGNAL(rowsInserted(const QModelIndex&, int, int)), this, SIGNAL(modelReset()));
//        QObject::connect(_pParentModel.data(), SIGNAL(rowsRemoved(const QModelIndex&, int, int)), this, SIGNAL(modelReset()));
//        QObject::connect(_pParentModel.data(), SIGNAL(rowsMoved(const QModelIndex&, int, int, const QModelIndex&, int)), this, SIGNAL(modelReset()));

//        QObject::connect(_pParentModel.data(), SIGNAL(columnsInserted(const QModelIndex&, int, int)), this, SIGNAL(modelReset()));
//        QObject::connect(_pParentModel.data(), SIGNAL(columnsRemoved(const QModelIndex&, int, int)), this, SIGNAL(modelReset()));
//        QObject::connect(_pParentModel.data(), SIGNAL(columnsMoved(const QModelIndex&, int, int, const QModelIndex&, int)), this, SIGNAL(modelReset()));
    }
    else
    {
        QObject::connect(m_parentModel.data(), &AQmlRemoteObjectHelperHeaderModel::rowsAboutToBeInserted, this, [this](){this->beginResetModel();});
        QObject::connect(m_parentModel.data(), &AQmlRemoteObjectHelperHeaderModel::rowsInserted, this, [this](){this->endResetModel();});
        QObject::connect(m_parentModel.data(), &AQmlRemoteObjectHelperHeaderModel::rowsAboutToBeRemoved, this, [this](){this->beginResetModel();});
        QObject::connect(m_parentModel.data(), &AQmlRemoteObjectHelperHeaderModel::rowsRemoved, this, [this](){this->endResetModel();});
        QObject::connect(m_parentModel.data(), &AQmlRemoteObjectHelperHeaderModel::rowsAboutToBeMoved, this, [this](){this->beginResetModel();});
        QObject::connect(m_parentModel.data(), &AQmlRemoteObjectHelperHeaderModel::rowsMoved, this, [this](){this->endResetModel();});
//        QObject::connect(_pParentModel.data(), &AQmlRemoteObjectHelperHeaderModel::modelAboutToBeReset, this, [this](){beginResetModel();});
//        QObject::connect(_pParentModel.data(), &AQmlRemoteObjectHelperHeaderModel::modelReset, this, [this](){endResetModel();});

//        QObject::connect(_pParentModel.data(), SIGNAL(rowsInserted(const QModelIndex&, int, int)), this, SIGNAL(modelReset()));
//        QObject::connect(_pParentModel.data(), SIGNAL(rowsRemoved(const QModelIndex&, int, int)), this, SIGNAL(modelReset()));
//        QObject::connect(_pParentModel.data(), SIGNAL(rowsMoved(const QModelIndex&, int, int, const QModelIndex&, int)), this, SIGNAL(modelReset()));

//        QObject::connect(_pParentModel.data(), SIGNAL(columnsInserted(const QModelIndex&, int, int)), this, SIGNAL(modelReset()));
//        QObject::connect(_pParentModel.data(), SIGNAL(columnsRemoved(const QModelIndex&, int, int)), this, SIGNAL(modelReset()));
//        QObject::connect(_pParentModel.data(), SIGNAL(columnsMoved(const QModelIndex&, int, int, const QModelIndex&, int)), this, SIGNAL(modelReset()));
    }
}

AQmlRemoteObjectHelperHeaderModel::~AQmlRemoteObjectHelperHeaderModel()
{
}

QHash<int, QByteArray> AQmlRemoteObjectHelperHeaderModel::roleNames() const
{
    return m_parentModel->roleNames();
}

int AQmlRemoteObjectHelperHeaderModel::rowCount(const QModelIndex &index) const
{
    Q_UNUSED(index);
    if( Qt::Vertical == m_orientation)
        return m_parentModel->rowCount();
    else
        return m_parentModel->columnCount();
}
QVariant AQmlRemoteObjectHelperHeaderModel::data(const QModelIndex &index, int role) const
{
    return m_parentModel->headerData(index.row(),m_orientation, role);
}

void AQmlRemoteObjectHelperHeaderModel::handleHeaderDataChanged_slot(Qt::Orientation ori, int nFirst, int nLast)
{
    if( ori == m_orientation)
    {
        emit dataChanged(index(nFirst,0),index(nLast,0), QVector<int>() << Qt::DisplayRole << m_parentModel->roleNames().keys().toVector());
    }
}

void AQmlRemoteObjectHelperHeaderModel::handleBeginReset_slot()
{
    beginResetModel();
}

void AQmlRemoteObjectHelperHeaderModel::handleEndReset_slot()
{
    endResetModel();
//    for (int i = 0; i < rowCount(); ++i)
//        qDebug() << QString("Column%1 = ").arg(i) << data(index(i,0), Qt::DisplayRole);
}

