import QtQuick 2.15
import QtQuick.Window 2.15
import QtQuick.Controls 2.15
import QtQuick.Layouts 1.15
import QtGraphicalEffects 1.15



ApplicationWindow {
    id: appwindow
    visible: true
    width: 1500
    height: 720
    title: qsTr("Remote Object Table Model Test")

    header: ToolBar {
        id: toolbar
        RowLayout {
            anchors.fill: parent
            anchors.rightMargin: 6
            Switch {
                id: swUpdate
                checked: true
                text: (checked ? "<font color='#088A29'>" + qsTr("Toggle to chars") + "</font>" : qsTr("Toggle to numbers"))
                onToggled: testcontroler.reloadData(checked)
            }
            TextField {
                id: filter
                Layout.fillWidth: true
                placeholderText: qsTr("Filter")
                onTextEdited: {
                    console.log("Calling testcontroler.filter_slot");
                    testcontroler.filter_slot( filter.text);
                }
            }

            Item {
                Layout.fillWidth: true
            }
        }
    }

    TableView {
        id: tvDetails

        implicitHeight: 200
        height: 200
        clip: true;
        columnSpacing: 2; rowSpacing: 2
        visible: true

        anchors.fill: parent

        columnWidthProvider: function (column) { return 150; }
        rowHeightProvider: function (column) { return 30; }
        topMargin: detailsColumnsHeader.implicitHeight
        model: testmodel
        delegate: Item {
            Rectangle {
                color: "#EEE"
                anchors.fill: parent
                clip: true;
                Text {
                    text: display ? display : "undef"
                    anchors.fill: parent
                    anchors.margins: 10

                    color: '#555555'
                    font.pixelSize: 15
                    verticalAlignment: Text.AlignVCenter
                    horizontalAlignment: Text.AlignRight
                }
            }
        }

        Row {
            id: detailsColumnsHeader
            y: tvDetails.contentY
            z: 2
            Repeater {
                id: repeaterHorz
                model: testmodelHorzHeader
                Label {
                    width: tvDetails.columnWidthProvider(index)+2
                    height: 30
                    text: display
                    color: '#aaaaaa'
                    font.pixelSize: 15
                    padding: 10
                    horizontalAlignment: Text.AlignHCenter
                    verticalAlignment: Text.AlignVCenter

                    background: Rectangle { color: "#333333" }

                    MouseArea {
                        id: horzHeaderArea
                        anchors.fill: parent
                        acceptedButtons: Qt.LeftButton | Qt.RightButton
                        onClicked: {
                            console.log("onClicked headercolumn: " + index +" , label: " + display)
                            if (mouse.button == Qt.LeftButton)
                            {
                                console.log("calling sort on column: " + index)
                                testcontroler.sort_slot(index);
                            }
                        }
                    }
                }
            }
        }
        ScrollBar.vertical: ScrollBar {
            id: vertDetailScrollBar
        }
        ScrollBar.horizontal: ScrollBar {
            id: horzDetailScrollBar

        }
    }
}
